/**
 * Js for the Advanced Sidebar Menu Pro widgets (front-end)
 *
 * @author Mat Lipe
 *
 * @since 1.5
 *
 */
/* global asm_pro, asm_pro_config, jQuery */
var asm_pro = window.asm_pro || {
	enabled_accordions: {},
	accordion: {}
};
var asm_pro_config = window.asm_pro_config || {
	accordion_styles : {
		hamburger: {
			expand: '',
			contract: ''
		}
	},
	accordion_style_key: ''
};

(function( $, a, config ){
	a.accordion = {
		init : function(){
			this._render_accordions();
		},

		/**
		 * Is a checkbox from a widget of a key checked.
		 * Done this way to allow for using asm_debug overrides to uncheck options.
		 *
		 * @since 3.10.1
		 *
		 * @param {string} $widget_id
		 * @param {string} $option
		 * @returns {*|boolean}
		 */
		isChecked : function( $widget_id, $option ) {
			return a.enabled_accordions[ $widget_id ][ $option ] && a.enabled_accordions[ $widget_id ][ $option ] === 'checked';
		},

		/**
		 * Iterate through the registered accordions
		 * sets up the icons
		 * hides the child menus
		 *
		 * @uses a.enabled_accordions
		 *
		 * @private
		 */
		_render_accordions : function(){
			var self = a.accordion;
			var parents = [];

			for( var $widget_id in a.enabled_accordions ){
				if( !a.enabled_accordions.hasOwnProperty( $widget_id ) ){
					continue;
				}

				var el = $( '[data-js="' + $widget_id + '"]' );
				// the .has() only target elements that have actual children in.
				// case of 'always display child pages' not being active.
				if ( a.accordion.isChecked( $widget_id, 'include_parent' ) && ! a.accordion.isChecked( $widget_id, 'include_parent_in_accordion' ) ) {
					parents = el.find( '> ul > li .has_children' ).has( 'ul' );
				} else {
					parents = el.find( 'ul .has_children' ).has( 'ul' );
				}


				parents.each( function(){
					var _parent_menu = $( this );
					var _icon = {};
					// current page and parents of current page should be open.
					if ( ! a.accordion.isChecked( $widget_id, 'keep_closed' )
						&& (
						_parent_menu.hasClass( 'current_page_item' )
						|| _parent_menu.hasClass( 'current-cat' )
						|| _parent_menu.hasClass( 'current-menu-item' )
						|| _parent_menu.find( '.current-cat' ).length
						|| _parent_menu.find( '.current_page_item' ).length
						|| _parent_menu.find( '.current-menu-item' ).length
						)
					){
						_parent_menu
							.addClass( 'open' )
							.find( '> ul' )
								.attr( 'aria-hidden', 'false' )
						_icon = self._get_icon( $widget_id, 'contract', _parent_menu );
						//everything else should be closed
					} else {
						_parent_menu
							.addClass( 'closed' )
							.find( '> ul' )
								.attr( 'aria-hidden', 'true' )
								.hide();
						_icon = self._get_icon( $widget_id, 'expand', _parent_menu );
					}

					if ( ! _parent_menu.find( '> span' ).length ) {
						_parent_menu.find( '> a' ).append( _icon );
					}
				} );

			}
		},

		/**
		 * Icon was clicked. We are either going to expand
		 * or contract a menu based on the icon's state.
		 * After the menu is changed the icon will change
		 * to the new one here.
		 *
		 * @param {JQuery<HTMLLIElement>} parent_menu - menu we are working with
		 * @param {JQuery<HTMLSpanElement>} icon - icon which was clicked
		 *
		 * @private
		 */
		_expand_sub_menu : function( parent_menu, icon ){
			var _icon = '';
			if( icon.data( 'state' ) === 'expand' ){
				parent_menu
					.addClass( 'open' )
					.removeClass( 'closed' );

				parent_menu.find( '> ul' )
					.attr( 'aria-hidden', 'false' )
					.slideDown();
				_icon = this._get_icon( icon.data( 'widget_id' ), 'contract', parent_menu );
				icon.replaceWith( _icon );
				_icon.focus();
			} else {
				parent_menu
					.addClass( 'closed' )
					.removeClass( 'open' );

				parent_menu.find( '> ul' )
					.attr( 'aria-hidden', 'true' )
					.slideUp();
				_icon = this._get_icon( icon.data( 'widget_id' ), 'expand', parent_menu );
				icon.replaceWith( _icon );
				_icon.focus();
			}
		},


		/**
		 *
		 * @param {string} $widget_id
		 * @param {string} $state - expand|contract
		 * @param {JQuery<HTMLLIElement>} parent_menu - the menu we are working with
		 *
		 * @since 3.12.0 (Support triggering expand/contract via enter key from screen reader)
		 *
		 * @returns {JQuery<HTMLSpanElement>}
		 * @private
		 */
		_get_icon : function( $widget_id, $state, parent_menu ){
			var $style = a.enabled_accordions[ $widget_id ][ config.accordion_style_key ];
			var icon = $( config.accordion_styles[ $style ][ $state ] );
			icon.data( 'state', $state );
			icon.data( 'widget_id', $widget_id );

			icon.on( 'click', function( ev ) {
				ev.preventDefault();
				a.accordion._expand_sub_menu( parent_menu, icon )
			} );
			// Enter key pressed by screen readers.
			icon.on( 'keypress', function( ev ) {
				if ( '13' === ev.keyCode ? ev.keyCode : ev.which ) {
					ev.preventDefault();
					a.accordion._expand_sub_menu( parent_menu, icon );
				}
			} );
			return icon;
		}

	};

	$( function(){
		a.accordion.init();

		setTimeout( function() {
			// Enables accordions when editing Elementor.
			if ( 'undefined' !== typeof elementorFrontend ) {
				elementorFrontend.hooks.addAction( 'frontend/element_ready/widget', function() {
					a.accordion.init();
				} );
			}

			// Enables accordions when editing Beaver Builder.
			$( '.fl-builder-content' ).on( 'fl-builder.layout-rendered', function() {
				a.accordion.init()
			} );
		}, 1000 );

		// Event on accordions loaded @since 3.4.1.
		$( document ).trigger( 'advanced-sidebar-menu-pro/accordions/loaded', [a.accordion, a] );
	} );

	// Event to pre interact with accordions @since 3.4.4.
	$( document ).trigger( 'advanced-sidebar-menu-pro/accordions/available', [a.accordion, a] );

})( jQuery, asm_pro, asm_pro_config );
