<?php

namespace Advanced_Sidebar_Menu;

use Advanced_Sidebar_Menu\Traits\Singleton;
use Advanced_Sidebar_Menu\Widget\Category;
use Advanced_Sidebar_Menu\Widget\Navigation_Menu;
use Advanced_Sidebar_Menu\Widget\Page;
use Advanced_Sidebar_Menu\Widget_Options\Shared\Styles;
use WP_Widget;

/**
 * Main widget factory.
 */
class Factory {
	use Singleton;

	const CSS_SELECTOR_APPEND = '-wrap';

	/**
	 * Increment the widget id using the number of widgets.
	 *
	 * @var int
	 */
	protected $widget_count = 0;


	/**
	 * Actions and filters.
	 */
	protected function hook() {
		remove_action( 'advanced-sidebar-menu/widget/page/after-form', 'advanced_sidebar_menu_widget_docs', 99 );
		remove_action( 'advanced-sidebar-menu/widget/category/after-form', 'advanced_sidebar_menu_widget_docs', 99 );
		add_action( 'advanced-sidebar-menu/widget/page/after-form', [ $this, 'add_documentation_link' ], 99, 4 );
		add_action( 'advanced-sidebar-menu/widget/category/after-form', [ $this, 'add_documentation_link' ], 99, 4 );
		add_action( 'advanced-sidebar-menu/widget/navigation-menu/after-form', [
			$this,
			'add_documentation_link',
		], 99, 4 );

		add_action( 'advanced-sidebar-menu/widget/page/before-form', [ $this, 'wrap_widget_form' ], 1, 2 );
		add_action( 'advanced-sidebar-menu/widget/page/after-form', [ $this, 'close_div' ], 99, 0 );
		add_action( 'advanced-sidebar-menu/widget/category/before-form', [ $this, 'wrap_widget_form' ], 1, 2 );
		add_action( 'advanced-sidebar-menu/widget/category/after-form', [ $this, 'close_div' ], 99, 0 );
		add_action( 'advanced-sidebar-menu/widget/navigation-menu/before-form', [ $this, 'wrap_widget_form' ], 1, 2 );
		add_action( 'advanced-sidebar-menu/widget/navigation-menu/after-form', [ $this, 'close_div' ], 99, 0 );

		add_filter( 'advanced-sidebar-menu/menus/page/output', [ $this, 'wrap_page_widget' ], 10, 3 );
		add_filter( 'advanced-sidebar-menu/menus/category/output', [ $this, 'wrap_widget_on_render' ], 10, 2 );
		add_filter( 'advanced-sidebar-menu/widget/navigation-menu/output', [ $this, 'wrap_widget_on_render' ], 10, 2 );

		add_action( 'widgets_init', [ $this, 'register_widgets' ] );
		add_action( 'advanced-sidebar-menu-pro/updates/loaded', [ $this, 'prevent_conflicts' ] );
	}


	/**
	 * Register the navigation menu widget.
	 *
	 * @action widgets_init 10 0
	 *
	 * @since  3.3.0
	 *
	 * @return void
	 */
	public function register_widgets() {
		register_widget( Navigation_Menu::class );
	}


	/**
	 * Are we a category widget or a page widget?
	 *
	 * @param WP_Widget $widget - Current widget class.
	 *
	 * @return string
	 * @deprecated
	 */
	public function get_widget_type( WP_Widget $widget ) {
		_deprecated_function( __METHOD__, '8.2.0' );
		if ( Category::NAME === $widget->id_base ) {
			return 'category';
		}

		return 'page';
	}


	/**
	 * Wrap the admin widget, so we can target with JS
	 *
	 * @param array      $instance - Widget settings.
	 * @param \WP_Widget $widget   - Widget class.
	 *
	 * @since 3.2.0
	 *
	 * @return void
	 */
	public function wrap_widget_form( array $instance, $widget ) {
		// because new widgets will have an __i__ which gets changed by js
		// this is the only way to track new widgets.
		$this->widget_count ++;
		$id = $widget->id_base . '-' . $this->widget_count;
		?>
		<div id="<?php echo esc_attr( $id ); ?>">
		<!-- done this way to handle reloading the widget after hitting save -->
		<script type="text/javascript">
			//new widgets being added
			if ( typeof ( asm_pro ) !== 'undefined' && typeof ( asm_pro.init ) === 'function' ) {
				asm_pro.init( '#<?php echo esc_js( $id ); ?>' );
			} else {
				//existing widgets
				//we don't trigger the init for the background __i__ widget on page load
				//because the color pickers will be broken when adding a new widget
				<?php if ( '__i__' !== $widget->number ) { ?>
				jQuery( function() {
					if ( typeof ( asm_pro ) !== 'undefined' ) {
						asm_pro.init( '#<?php echo esc_js( $id ); ?>' );
					}
				} );
				<?php } ?>
			}
		</script>
		<?php
	}


	/**
	 * Close a div via </div>
	 *
	 * @since 3.2.0
	 *
	 * @return void
	 */
	public function close_div() {
		?>
		</div>
		<?php
	}


	/**
	 * Wrap widget, so we can target it directly with js and css and
	 * not worry about themes which do not declare the widget id
	 *
	 * @notice Ugh! THE page widget has different args, so it needs its own method.
	 *
	 * @param string   $output - Widget output.
	 * @param \WP_Post $post   - Current post.
	 * @param array    $args   - Widget arguments.
	 *
	 * @since  1.8.0
	 *
	 * @return string
	 */
	public function wrap_page_widget( $output, $post, $args ) {
		return $this->widget_wrap( $output, $args );
	}


	/**
	 * Wrap widget, so we can target it directly with js and css and
	 * not worry about themes which do not declare the widget id
	 *
	 * @notice Ugh! THE page widget has different args, so it needs its own method
	 *
	 * @param string $output - Widget output.
	 * @param array  $args   - Widget arguments.
	 *
	 * @since  3.3.0
	 *
	 * @return string
	 */
	public function wrap_widget_on_render( $output, $args ) {
		return $this->widget_wrap( $output, $args );
	}


	/**
	 * Wrap all widgets, so they may be targeted with JS and CSS.
	 *
	 * @param string $output - Widget output.
	 * @param array  $args   - Widget arguments.
	 *
	 * @since  3.11.0
	 *
	 * @return string
	 */
	protected function widget_wrap( $output, $args ) {
		return '<div class="advanced-sidebar-menu-wrap" id="' . $args['widget_id'] . self::CSS_SELECTOR_APPEND . '" data-js="' . $args['widget_id'] . self::CSS_SELECTOR_APPEND . '">' . $output . '</div>';
	}


	/**
	 * Add a link to documentation
	 *
	 * @param array      $instance - Widget settings.
	 * @param \WP_Widget $widget   - Widget class.
	 *
	 * @since 3.2.0
	 *
	 * @return void
	 */
	public function add_documentation_link( $instance, $widget ) {
		?>
		<p style="text-align: right">
			<a
				href="<?php echo esc_url( $this->get_docs_url( $widget ) ); ?>"
				target="_blank"
				rel="noopener noreferrer">
				<?php esc_html_e( 'widget documentation', 'advanced-sidebar-menu' ); ?>
			</a>
		</p>
		<?php
	}


	/**
	 * Get URL to documentation based on provided widget.
	 *
	 * @param \WP_Widget $widget - Widget class.
	 *
	 * @static
	 *
	 * @return string
	 */
	public function get_docs_url( $widget ) {
		switch ( $widget->id_base ) {
			case Category::NAME:
				return 'https://onpointplugins.com/advanced-sidebar-menu/advanced-sidebar-menu-pro-widget-docs/advanced-sidebar-menu-pro-categories-menu/';
			case Page::NAME:
				return 'https://onpointplugins.com/advanced-sidebar-menu/advanced-sidebar-menu-pro-widget-docs/advanced-sidebar-menu-pro-pages-widget/';
			case Navigation_Menu::NAME:
				return 'https://onpointplugins.com/advanced-sidebar-menu/advanced-sidebar-menu-pro-widget-docs/advanced-sidebar-menu-pro-navigation-menu/';
		}

		return '';
	}


	/**
	 * Prevent conflicts by cleaning up the widget settings and adding a warning message.
	 *
	 * @param ?string $message - Warning message.
	 *
	 * @since 8.1.3
	 */
	public function prevent_conflicts( $message ) {
		if ( empty( $message ) ) {
			return;
		}

		// Disable auto updates which won't work.
		add_filter( 'auto_update_plugin', function ( $update, $item ) {
			return ( 'advanced-sidebar-menu-pro' === $item->slug ) ? false : $update;
		}, 10, 2 );

		add_action( 'widgets_init', function () use ( $message ) {
			remove_all_actions( 'advanced-sidebar-menu/widget/category/left-column' );
			remove_all_actions( 'advanced-sidebar-menu/widget/page/left-column' );
			remove_all_actions( 'advanced-sidebar-menu/widget/navigation-menu/left-column' );
			remove_all_actions( 'advanced-sidebar-menu/widget/category/right-column' );
			remove_all_actions( 'advanced-sidebar-menu/widget/page/right-column' );
			remove_all_actions( 'advanced-sidebar-menu/widget/navigation-menu/right-column' );
			remove_action( 'advanced-sidebar-menu/widget/category/after-form', [
				Styles::instance(),
				'render_fields',
			] );
			remove_action( 'advanced-sidebar-menu/widget/page/after-form', [ Styles::instance(), 'render_fields' ] );
			remove_action( 'advanced-sidebar-menu/widget/navigation-menu/after-form', [
				Styles::instance(),
				'render_fields',
			] );

			$render = function () use ( $message ) {
				?>
				<div class="advanced-sidebar-menu-column-box" style="border-color: red">
					<p>
						<?php echo $message; //phpcs:ignore ?>
					</p>
				</div>
				<?php
			};

			add_action( 'advanced-sidebar-menu/widget/category/after-form', $render );
			add_action( 'advanced-sidebar-menu/widget/page/after-form', $render );
			add_action( 'advanced-sidebar-menu/widget/navigation-menu/after-form', $render );
		} );
	}

}

