<?php

namespace Advanced_Sidebar_Menu\Menus;

use Advanced_Sidebar_Menu\Nav_Menu\Objects;
use Advanced_Sidebar_Menu\Widget\Navigation_Menu;

/**
 * Navigation Menu.
 *
 * @author OnPoint Plugins
 * @since  3.3.0
 */
class Navigation extends Menu_Abstract {
	const HOOK_PRIORITY = 10;


	/**
	 * Add classes to menu items.
	 *
	 * @see  Navigation::remove_hooks()
	 *
	 * @return void
	 */
	public function add_hooks() {
		add_filter( 'nav_menu_submenu_css_class', [ $this, 'add_child_sidebar_menu_class' ], self::HOOK_PRIORITY, 3 );

		add_filter( 'wp_nav_menu_objects', [ $this, 'add_has_children_class' ], self::HOOK_PRIORITY );
		add_filter( 'wp_nav_menu_objects', [ $this, 'exclude_non_family_items' ], self::HOOK_PRIORITY );
	}


	/**
	 * Remove hooks which add classes to menu items to keep the changes contained to the widget.
	 *
	 * @see  Navigation::add_hooks()
	 *
	 * @return void
	 */
	protected function remove_hooks() {
		remove_filter( 'nav_menu_submenu_css_class', [ $this, 'add_child_sidebar_menu_class' ], self::HOOK_PRIORITY );
		remove_filter( 'wp_nav_menu_objects', [ $this, 'add_has_children_class' ], self::HOOK_PRIORITY );
		remove_filter( 'wp_nav_menu_objects', [ $this, 'exclude_non_family_items' ], self::HOOK_PRIORITY );
	}


	/**
	 * Is this widget being displayed?
	 *
	 * @return bool
	 */
	public function is_displayed() {
		$display = false;
		if ( ! empty( $this->instance[ Navigation_Menu::NAVIGATION_MENU ] ) ) {
			$nav_menu = wp_get_nav_menu_object( $this->instance[ Navigation_Menu::NAVIGATION_MENU ] );

			if ( ! empty( $nav_menu ) ) {
				$display = true;
			}
			if ( $this->checked( Navigation_Menu::CURRENT_PARENT_ONLY ) && ! $this->checked( Navigation_Menu::DISPLAY_NO_CURRENT ) ) {
				$menu_items = wp_get_nav_menu_items( $nav_menu );
				_wp_menu_item_classes_by_context( $menu_items );
				$objects = Objects::factory( $menu_items );
				$display = (bool) $objects->get_current_item();
			}
		}

		return apply_filters( 'advanced-sidebar-menu/menus/navigation/is-displayed', $display, $this->args, $this->instance, $this );
	}


	/**
	 * Exclude any items not part of the current menu item's family, if settings say to do so
	 *
	 * @param array $menu_items - List of menu items which will be supplied to `wp_nav_menu`.
	 *
	 * @since 3.7.0 - Run every menu through flatten which excludes needed items.
	 *
	 * @since 3.3.1
	 * @return array
	 */
	public function exclude_non_family_items( array $menu_items ) {
		$objects = Objects::factory( $menu_items );
		if ( ! $this->checked( Navigation_Menu::CURRENT_PARENT_ONLY ) ) {
			return $objects->flatten();
		}

		$current = $objects->get_current_item();

		// Not viewing a menu item and want the top level to display.
		if ( false === $current && $this->checked( Navigation_Menu::DISPLAY_NO_CURRENT ) ) {
			$objects->remove_all_child_items();

			return $objects->flatten();
		}

		if ( false !== $current && $this->checked( Navigation_Menu::TOP_LEVEL_SIBLINGS ) && false === $current->get_parent() ) {
			$count = count( $objects->get_current_item_family() );
			// No children in family so we display the top level items.
			if ( $count < 2 ) {
				$objects->remove_all_child_items();
			} else {
				// There are children available so we display the family only.
				$objects->remove_all_non_family_items();
			}

			return $objects->flatten();
		}

		// Default action when option is checked.
		$objects->remove_all_non_family_items();

		return $objects->flatten();
	}


	/**
	 * Add 'has_children' class to menu items which have children.
	 *
	 * @filter wp_nav_menu_objects 10 2
	 *
	 * @param array $menu_items - All menu items.
	 *
	 * @since  3.3.1
	 *
	 * @return array
	 */
	public function add_has_children_class( array $menu_items ) {
		foreach ( $menu_items as $_item ) {
			if ( in_array( 'menu-item-has-children', $_item->classes, true ) ) {
				$_item->classes[] = 'has_children';
			}
			$_item->classes = array_unique( $_item->classes );
		}

		return $menu_items;
	}


	/**
	 * Add 'child-sidebar-menu' or 'children' classes to menu items.
	 *
	 * @filter nav_menu_submenu_css_class 10 3
	 *
	 * @param array $classes - List of existing classes.
	 *
	 * @param array $args    - Menu arguments.
	 * @param int   $depth   - Depth of menu.
	 *
	 * @return array
	 */
	public function add_child_sidebar_menu_class( array $classes, $args, $depth ) {
		if ( 0 === $depth ) {
			return [ 'child-sidebar-menu' ];
		}

		return [ 'children' ];
	}


	/**
	 * Get the arguments to be sent to wp_nav_menu
	 *
	 * @return array
	 */
	public function get_nav_menu_args() {
		$_args = [
			'fallback_cb' => '',
			'menu'        => wp_get_nav_menu_object( $this->instance[ Navigation_Menu::NAVIGATION_MENU ] ),
			'echo'        => false,
			'container'   => false,
			'menu_class'  => 'parent-sidebar-menu',
		];

		return apply_filters( 'advanced-sidebar-menu/menus/navigation/get-nav-menu-args', $_args, $this );
	}


	/**
	 * Render the menu.
	 */
	public function render() {
		if ( ! $this->is_displayed() ) {
			return;
		}

		$this->add_hooks();

		//phpcs:disable WordPress.Security.EscapeOutput.OutputNotEscaped
		echo $this->args['before_widget'];

		do_action( 'advanced-sidebar-menu/menus/navigation/render', $this );

		$this->title();

		$output = wp_nav_menu( $this->get_nav_menu_args() );

		echo apply_filters( 'advanced-sidebar-menu/widget/navigation-menu/output', $output, $this->args, $this->instance, $this );

		do_action( 'advanced-sidebar-menu/menus/navigation-menu/render/after', $this );

		echo $this->args['after_widget'];

		$this->remove_hooks();
		//phpcs:enable WordPress.Security.EscapeOutput.OutputNotEscaped
	}


	/**
	 * CURRENTLY NOT APPLICABLE
	 *
	 * @phpstan-ignore-next-line
	 */
	public function get_top_parent_id() {
	}


	/**
	 * CURRENTLY NOT APPLICABLE
	 *
	 * @phpstan-ignore-next-line
	 */
	public function get_order_by() {
	}


	/**
	 * CURRENTLY NOT APPLICABLE
	 *
	 * @phpstan-ignore-next-line
	 */
	public function get_order() {
	}


	/**
	 * We currently have not control over this so 100 should cover all.
	 * Right now just used for Block styles.
	 *
	 * @return int
	 */
	public function get_levels_to_display() {
		return 100;
	}

}
