<?php

namespace Advanced_Sidebar_Menu\Nav_Menu;

/**
 * Nav Menu Item
 *
 * @author OnPoint Plugins
 * @since  3.3.3
 */
class Item {

	/**
	 * Is this item the menu item which matches the current page?
	 *
	 * @var bool
	 */
	public $current;

	/**
	 * Is this item an ancestor of the menu item which
	 * matches the current page?
	 *
	 * @var bool
	 */
	public $current_item_ancestor;

	/**
	 * WP Nav post type item.
	 *
	 * @var \WP_Post
	 */
	protected $menu_item;

	/**
	 * ID of this menu item's parent.
	 *
	 * @var int
	 */
	public $menu_item_parent;

	/**
	 * Objects class representation of this menu.
	 *
	 * @var Objects
	 */
	protected $objects;


	/**
	 * Get the menu item's post ID.
	 *
	 * @return int
	 */
	public function get_id() {
		return (int) $this->menu_item->ID;
	}


	/**
	 * Get the id of post, page, category, etc. of
	 * this menu items linked item.
	 *
	 * If this is a custom link this id does not apply.
	 *
	 * @since 3.11.0
	 *
	 * @return int
	 *
	 * @property string $object_id - Special property added by `wp_setup_nav_menu_item`.
	 */
	public function get_object_id() {
		// @phpstan-ignore-next-line
		return (int) $this->menu_item->object_id;
	}


	/**
	 * Get the menu item.
	 *
	 * @since 3.11.0
	 *
	 * @return \WP_Post
	 */
	public function get_menu_item() {
		return $this->menu_item;
	}


	/**
	 * Is this post a parent, child, or grandchild of
	 * the current page's top-level parent.
	 *
	 * @return bool
	 */
	public function is_in_current_family() {
		return $this->objects->is_in_current_family( $this->get_id() );
	}


	/**
	 * Get all direct children of this menu item.
	 *
	 * @return Item[]
	 */
	public function get_children() {
		return $this->objects->get_child_items( $this->menu_item->ID );
	}


	/**
	 * Return the parent menu item of this item.
	 *
	 * @return bool|Item
	 */
	public function get_parent() {
		if ( 0 === (int) $this->menu_item_parent ) {
			return false;
		}

		return $this->objects->get_item( $this->menu_item_parent );
	}


	/**
	 * Is this menu item a link to a page?
	 *
	 * @since 3.7.0
	 *
	 * @return bool
	 *
	 * @property string $object - Special property added by `wp_setup_nav_menu_item`.
	 */
	public function is_page() {
		// @phpstan-ignore-next-line
		return ( 'page' === $this->menu_item->object );
	}


	/**
	 * If this item or a parent/grandparent of this item is excluded
	 *
	 * @return bool
	 */
	public function is_excluded() {
		$excluded = in_array( $this->get_id(), $this->objects->get_excluded_ids(), true );

		if ( false === $excluded ) {
			$parent = $this->get_parent();
			if ( false !== $parent ) {
				$excluded = $parent->is_excluded();
			}
		}

		return $excluded;
	}


	/**
	 * Get Item instance based on a wp nav menu item, and the corresponding objects class.
	 *
	 * @param \WP_Post $menu_item - WP Nav menu item.
	 * @param Objects  $objects   - Object class representation of the menu.
	 *
	 * @see wp_setup_nav_menu_item
	 * @see _wp_menu_item_classes_by_context
	 *
	 * @return Item
	 */
	public static function factory( \WP_Post $menu_item, Objects $objects ) {
		$item = new static(); // @phpstan-ignore-line
		$item->menu_item = $menu_item;
		$item->objects = $objects;

		// Special properties from `wp_setup_nav_menu_item` && `_wp_menu_item_classes_by_context`.
		$item->current = $menu_item->current; // @phpstan-ignore-line
		$item->current_item_ancestor = $menu_item->current_item_ancestor; // @phpstan-ignore-line
		$item->menu_item_parent = $menu_item->menu_item_parent; // @phpstan-ignore-line

		return $item;
	}
}
