<?php

namespace Advanced_Sidebar_Menu\Widget_Options\Category;

use Advanced_Sidebar_Menu\Menus\Category;
use Advanced_Sidebar_Menu\Traits\Singleton;
use Advanced_Sidebar_Menu\Walkers\Posts;
use Advanced_Sidebar_Menu\Widget\Widget_Abstract;
use Advanced_Sidebar_Menu\Widget_Options\Widget_Options_Abstract;

/**
 * Display posts widget option.
 *
 * @author OnPoint Plugins
 * @since  3.2.0
 */
class Display_Posts extends Widget_Options_Abstract {
	use Singleton;

	const NAME = 'display-posts';

	const POST_TYPE = 'display-posts/post-type';
	const LIMIT     = 'display-posts/limit';

	const LEVELS_ALL                  = 'all';
	const LEVELS_CHILD                = 'child';
	const LEVELS_CHILD_AND_GRANDCHILD = 'checked'; // Mimic old checkbox option.
	const LEVELS_GRANDCHILD           = 'grandchild';
	const LEVELS_PARENT               = 'parent';
	const LEVELS_PARENT_AND_CHILD     = 'parent-child';

	/**
	 * Default values for the widget options.
	 *
	 * @var array
	 */
	protected $defaults = [
		self::NAME      => '',
		self::POST_TYPE => 'post',
		self::LIMIT     => 20,
	];


	/**
	 * Add actions and filters.
	 *
	 * @return void
	 */
	protected function hook() {
		add_action( 'advanced-sidebar-menu/widget/category/left-column', [ $this, 'render_fields' ], 6, 2 );

		add_filter( 'advanced-sidebar-menu/menus/category/get-list-categories-args', [
			$this,
			'use_display_posts_walker',
		], 11, 3 );
	}


	/**
	 * Use the custom walker which displays posts under terms.
	 * Apply only to levels which are selected to do so.
	 *
	 * @param array    $args  - Args passed to wp_list_categories().
	 * @param string   $level - The current menu level.
	 * @param Category $menu  - The menu class for the category widget.
	 *
	 * @return array
	 */
	public function use_display_posts_walker( $args, $level, $menu ) {
		$instance = $menu->get_widget_instance();
		$args['level'] = $level;
		if ( empty( $instance[ self::NAME ] ) ) {
			return $args;
		}

		switch ( true ) {
			case self::LEVELS_ALL === $instance[ self::NAME ] || Category::LEVEL_DISPLAY_ALL === $level:
			case self::LEVELS_PARENT === $instance[ self::NAME ] && Category::LEVEL_PARENT === $level:
			case self::LEVELS_CHILD === $instance[ self::NAME ] && Category::LEVEL_CHILD === $level:
			case self::LEVELS_GRANDCHILD === $instance[ self::NAME ] && Category::LEVEL_GRANDCHILD === $level:
			case self::LEVELS_PARENT_AND_CHILD === $instance[ self::NAME ] && ( Category::LEVEL_PARENT === $level || Category::LEVEL_CHILD === $level ):
			case self::LEVELS_CHILD_AND_GRANDCHILD === $instance[ self::NAME ] && ( Category::LEVEL_CHILD === $level || Category::LEVEL_GRANDCHILD === $level ):
				$args['walker'] = new Posts( $menu->get_widget_instance() );
				break;
		}

		return $args;
	}


	/**
	 * Get the plural form of the select post type.
	 *
	 * @param array $instance - Widget settings.
	 *
	 * @return string
	 */
	public function get_post_types_label( array $instance ) {
		if ( empty( $instance[ self::POST_TYPE ] ) ) {
			return 'posts';
		}
		$pt = get_post_type_object( $instance[ self::POST_TYPE ] );
		if ( null === $pt ) {
			return 'posts';
		}

		return strtolower( $pt->labels->name );
	}


	/**
	 * The possible options for levels to display posts under.
	 *
	 * @param array $instance - Widget settings.
	 *
	 * @since 3.5.0
	 *
	 * @return array
	 */
	public function get_possible_level_options( array $instance ) {
		return [
			/* Translators: {taxonomy plural label} Label for selected taxonomy in plural form */
			self::LEVELS_ALL                  => sprintf( _x( 'all %1$s', '{taxonomy plural label}', 'advanced-sidebar-menu' ), strtolower( Taxonomy::instance()->get_taxonomy_label( $instance, false ) ) ),
			self::LEVELS_PARENT               => __( 'highest level parents', 'advanced-sidebar-menu' ),
			self::LEVELS_PARENT_AND_CHILD     => __( 'highest level parents and children', 'advanced-sidebar-menu' ),
			self::LEVELS_CHILD                => __( 'children', 'advanced-sidebar-menu' ),
			self::LEVELS_CHILD_AND_GRANDCHILD => __( 'children and grandchildren', 'advanced-sidebar-menu' ),
			self::LEVELS_GRANDCHILD           => __( 'grandchildren', 'advanced-sidebar-menu' ),

		];
	}


	/**
	 * Get available post types for this widget
	 *
	 * @param array $instance - Widget settings.
	 *
	 * @access public for unit testing
	 *
	 * @return array
	 */
	public function get_post_types( array $instance ) {
		$tax = get_taxonomy( Taxonomy::instance()->get_taxonomy( 'category', [], $instance ) );

		$post_types = [];
		$objects = get_post_types( [], 'objects' );
		foreach ( $objects as $_object ) {
			if ( in_array( $tax->name, $_object->taxonomies, true ) || in_array( $_object->name, $tax->object_type, true ) ) {
				$post_types[ $_object->name ] = $_object->labels->name;
			}
		}

		$post_types = apply_filters( 'advanced-sidebar-menu/widget/category/available-post-types', $post_types, $instance );

		return $post_types;
	}


	/**
	 * Render the widget fields.
	 *
	 * @param array           $instance - Widget settings.
	 * @param Widget_Abstract $widget   - Widget class.
	 *
	 * @return void
	 */
	public function render_fields( array $instance, $widget ) {
		$instance = (array) wp_parse_args( $instance, $this->defaults );
		?>
		<div class="advanced-sidebar-menu-column-box">
			<p>
				<label for="<?php echo esc_attr( $widget->get_field_id( self::NAME ) ); ?>">
					<?php
					/* translators: Selected {post type plural label} */
					printf( esc_html_x( 'Display assigned %1$s under', '{post type plural label}', 'advanced-sidebar-menu' ), esc_html( $this->get_post_types_label( $instance ) ) );
					?>
				</label>
				<select
					id="<?php echo esc_attr( $widget->get_field_id( self::NAME ) ); ?>"
					name="<?php echo esc_attr( $widget->get_field_name( self::NAME ) ); ?>"
					class="advanced-sidebar-menu-block-field"
					data-js="advanced-sidebar-menu-pro/widget-options/category/display-posts/levels"
					data-target="<?php echo esc_attr( $widget->get_field_id( 'display-posts/additional-fields' ) ); ?>">
					<option value="">
						<?php esc_html_e( '- none -', 'advanced-sidebar-menu' ); ?>
					</option>
					<?php
					foreach ( $this->get_possible_level_options( $instance ) as $_name => $_label ) {
						printf( '<option value="%s" %s">%s</option>', esc_attr( $_name ), selected( $_name, $instance[ self::NAME ] ), esc_html( $_label ) );
					}
					?>
				</select>
			</p>
			<div <?php echo empty( $instance[ self::NAME ] ) ? 'data-advanced-sidebar-menu-hide="1"' : ''; ?>
				id="<?php echo esc_attr( $widget->get_field_id( 'display-posts/additional-fields' ) ); ?>">
				<?php
				$post_types = $this->get_post_types( $instance );
				unset( $post_types['post'] );
				if ( ! empty( $post_types ) ) {
					?>
					<p>
						<label for="<?php echo esc_attr( $widget->get_field_id( self::POST_TYPE ) ); ?>">
							<?php esc_html_e( 'Post type', 'advanced-sidebar-menu' ); ?>
						</label>
						<select
							id="<?php echo esc_attr( $widget->get_field_id( self::POST_TYPE ) ); ?>"
							name="<?php echo esc_attr( $widget->get_field_name( self::POST_TYPE ) ); ?>">
							<option value="post">
								<?php esc_html_e( 'Posts', 'advanced-sidebar-menu' ); ?>
							</option>
							<?php
							foreach ( $post_types as $_name => $_label ) {
								printf( '<option value="%s" %s">%s</option>', esc_attr( $_name ), selected( $_name, $instance[ self::POST_TYPE ] ), esc_html( $_label ) );
							}
							?>
						</select>
					</p>
					<?php
				} else {
					?>
					<input
						name="<?php echo esc_attr( $widget->get_field_name( self::POST_TYPE ) ); ?>"
						type="hidden"
						value="post" />
					<?php
				}
				?>
				<p>
					<label for="<?php echo esc_attr( $widget->get_field_id( self::LIMIT ) ); ?>">
						<?php
						ob_start();
						?>
						<select
							id="<?php echo esc_attr( $widget->get_field_id( self::LIMIT ) ); ?>"
							name="<?php echo esc_attr( $widget->get_field_name( self::LIMIT ) ); ?>">
							<option value="100">
								<?php esc_html_e( '- All -', 'advanced-sidebar-menu' ); ?>
							</option>
							<?php
							for ( $i = 1; $i <= 25; $i ++ ) {
								printf( '<option value="%s" %s">%s</option>', esc_attr( (string) $i ), selected( $i, $instance[ self::LIMIT ] ), esc_html( (string) $i ) );
							}
							?>
						</select>
						<?php

						/* translators: {select html input}, {Selected post type plural label} */
						printf( esc_html__( 'Display %1$s %2$s', 'advanced-sidebar-menu' ), ob_get_clean(), esc_html( $this->get_post_types_label( $instance ) ) ); //phpcs:ignore WordPress.Security.EscapeOutput.OutputNotEscaped
						?>
					</label>
				</p>
			</div>
		</div>
		<?php
	}

}
