<?php

namespace Advanced_Sidebar_Menu\Widget_Options\Shared;

use Advanced_Sidebar_Menu\Menus\Menu_Abstract;
use Advanced_Sidebar_Menu\Pro_Scripts;
use Advanced_Sidebar_Menu\Traits\Singleton;
use Advanced_Sidebar_Menu\Widget\Navigation_Menu;
use Advanced_Sidebar_Menu\Widget\Widget_Abstract;
use Advanced_Sidebar_Menu\Widget_Options\Widget_Options_Abstract;

/**
 * Accordion widget settings.
 *
 * @author OnPoint Plugins
 * @since  1.5
 */
class Accordion extends Widget_Options_Abstract {
	use Singleton;

	const NAME = 'enable_accordion';

	const ENABLE_ACCORDION = self::NAME;
	const ACCORDION_STYLE  = 'accordion_style';
	const ICON_COLOR       = 'icon_color';
	const KEEP_CLOSED      = 'keep_closed';
	const INCLUDE_PARENT   = 'include_parent_in_accordion';

	/**
	 * Default widget setttings.
	 *
	 * @var array
	 */
	private $defaults = [
		self::ENABLE_ACCORDION        => 0,
		self::ACCORDION_STYLE         => 'plus',
		self::ICON_COLOR              => false,
		self::KEEP_CLOSED             => false,
		Menu_Abstract::INCLUDE_PARENT => false,
		self::INCLUDE_PARENT          => false,
	];


	/**
	 * Actions.
	 */
	protected function hook() {
		add_action( 'advanced-sidebar-menu/menus/page/render', [ $this, 'render' ] );
		add_action( 'advanced-sidebar-menu/menus/category/render', [ $this, 'render' ] );
		add_action( 'advanced-sidebar-menu/menus/navigation/render', [ $this, 'render' ] );

		add_action( 'advanced-sidebar-menu/widget/page/right-column', [ $this, 'render_fields' ], 20, 2 );
		add_action( 'advanced-sidebar-menu/widget/category/right-column', [ $this, 'render_fields' ], 20, 2 );
		add_action( 'advanced-sidebar-menu/widget/navigation-menu/right-column', [ $this, 'render_fields' ], 10, 2 );
	}


	/**
	 * Render the required js and css on the front end.
	 *
	 * @param Menu_Abstract $menu - menu class.
	 *
	 * @return void
	 */
	public function render( Menu_Abstract $menu ) {
		$instance = $menu->get_widget_instance();
		$widget_args = $menu->get_widget_args();
		if ( empty( $instance[ self::NAME ] ) ) {
			return;
		}

		wp_enqueue_style( 'dashicons' );
		Pro_Scripts::instance()->frontend_scripts();
		$this->inline_js( $menu );

		?>
		<style>
			#<?php echo esc_attr( $widget_args['widget_id'] ); ?>-wrap .dashicons {
				outline: none;
				cursor: pointer;
				<?php
				if ( empty( $instance[ Style_Targeting::BLOCK_STYLE ] ) ) {
					?>
				margin: 2px 5px 0;
					<?php
				}
				?>
			}
		</style>
		<?php

		if ( ! empty( $instance[ self::ICON_COLOR ] ) ) {
			?>
			<style>
				#<?php echo esc_attr( $widget_args['widget_id'] ); ?>-wrap .dashicons {
					color: <?php echo esc_attr( $instance[ self::ICON_COLOR ] ); ?> !important;
				}
			</style>
			<?php
		}
	}


	/**
	 * Register the accordion via inline JS.
	 *
	 * Done this way to account for Page Builders like Elementor.
	 *
	 * @param Menu_Abstract $menu - Current menu being processed.
	 */
	protected function inline_js( Menu_Abstract $menu ) {
		$settings = array_merge( $this->defaults, array_intersect_key( $menu->get_widget_instance(), $this->defaults ) );
		$widget_args = $menu->get_widget_args();
		?>
		<script>
			var asm_pro = window.asm_pro || {};
			<?php
			// This may be set elsewhere without `enabled_accordions` therefore
			// using || {enabled_accordions:{}} does not cover every case.
			?>
			if ( 'undefined' === typeof asm_pro.enabled_accordions ) {
				asm_pro.enabled_accordions = {};
			}
			asm_pro.enabled_accordions[ "<?php echo esc_js( $widget_args['widget_id'] ) . '-wrap"] = ' . wp_json_encode( $settings ); ?>;
		</script>
		<?php
	}


	/**
	 * Get available icon sets which will be used by accordion menus.
	 *
	 * @since 3.12.0 (Support screen readers tabbing and entering)
	 *
	 * @return array
	 */
	public function get_accordion_style_options() {
		$options = [
			'plus'        => [
				'label'    => __( 'Plus', 'advanced-sidebar-menu' ),
				'expand'   => '<span aria-expanded="false" tabindex="0" role="button" class="dashicons dashicons-plus"></span>',
				'contract' => '<span aria-expanded="true" tabindex="0" role="button" class="dashicons dashicons-minus"></span>',
			],
			'arrow'       => [
				'label'    => __( 'Arrow', 'advanced-sidebar-menu' ),
				'expand'   => '<span aria-expanded="false" tabindex="0" role="button" class="dashicons dashicons-arrow-down-alt2"></span>',
				'contract' => '<span aria-expanded="true" tabindex="0" role="button" class="dashicons dashicons-arrow-up-alt2"></span>',
			],
			'solid_arrow' => [
				'label'    => __( 'Solid Arrow', 'advanced-sidebar-menu' ),
				'expand'   => '<span aria-expanded="false" tabindex="0" role="button" class="dashicons dashicons-arrow-down"></span>',
				'contract' => '<span aria-expanded="true" tabindex="0" role="button" class="dashicons dashicons-arrow-up"></span>',
			],
			'hamburger'   => [
				'label'    => __( 'Hamburger', 'advanced-sidebar-menu' ),
				'expand'   => '<span aria-expanded="false" tabindex="0" role="button" class="dashicons dashicons-menu"></span>',
				'contract' => '<span aria-expanded="true" tabindex="0" role="button" class="dashicons dashicons-minus"></span>',
			],
		];

		return apply_filters( 'advanced-sidebar-menu/styles/accordion', $options );
	}


	/**
	 * Render the widget fields.
	 *
	 * @param array           $instance - widget settings.
	 * @param Widget_Abstract $widget   - widget class.
	 *
	 * @return void
	 */
	public function render_fields( array $instance, $widget ) {
		$instance = (array) wp_parse_args( $instance, $this->defaults );

		$style_options = $this->get_accordion_style_options();

		?>
		<div class="advanced-sidebar-menu-column-box">
			<p>
				<?php $widget->checkbox( self::ENABLE_ACCORDION, 'accordion-wrap' ); ?>
				<label>
					<?php esc_html_e( 'Enable accordion menu', 'advanced-sidebar-menu' ); ?>
				</label>
				<?php
				if ( Navigation_Menu::NAME !== $widget->id_base ) {
					?>
					<br />
					<span class="description">
						<em>
							<?php
							/* translators: Selected post type plural label */
							printf( esc_html__( 'Commonly used with "Always display child %s."', 'advanced-sidebar-menu' ), esc_html( strtolower( $this->get_object_type_label( $instance, $widget, false ) ) ) );
							?>
						</em>
					</span>
					<?php
				}
				?>
			</p>
			<div <?php $widget->hide_element( self::ENABLE_ACCORDION, 'accordion-wrap' ); ?>>
				<p>
					<label>
						<?php esc_html_e( 'Icon style', 'advanced-sidebar-menu' ); ?>
					</label>
					<select
						data-js="advanced-sidebar-menu-pro/widget/accordion-style"
						data-target="<?php echo esc_attr( $widget->get_field_id( 'accordion-icons' ) ); ?>"
						id="<?php echo esc_attr( $widget->get_field_id( self::ACCORDION_STYLE ) ); ?>"
						name="<?php echo esc_attr( $widget->get_field_name( self::ACCORDION_STYLE ) ); ?>"
						title="<?php esc_attr_e( 'Accordion icons style', 'advanced-sidebar-menu' ); ?>">
						<?php
						foreach ( $style_options as $_style => $_icons ) {
							printf( '<option value="%s" %s>%s</option>', esc_attr( $_style ), selected( $_style, $instance[ self::ACCORDION_STYLE ] ), esc_html( $_icons['label'] ) );
						}
						?>
					</select>
					<span data-js="<?php echo esc_attr( $widget->get_field_id( 'accordion-icons' ) ); ?>" class="advanced-sidebar-menu-icon-preview" style="<?php echo empty( $instance[ self::ICON_COLOR ] ) ? '' : esc_attr( "color: {$instance[ self::ICON_COLOR ]}" ); ?>">
						<?php
						echo wp_kses( $style_options[ $instance[ self::ACCORDION_STYLE ] ]['contract'] . ' ' . $style_options[ $instance[ self::ACCORDION_STYLE ] ]['expand'], [ 'span' => [ 'class' => [] ] ] );
						?>
					</span>
				</p>
				<p>
					<label for="<?php echo esc_attr( $widget->get_field_id( self::ICON_COLOR ) ); ?>">
						<?php esc_html_e( 'Icon color', 'advanced-sidebar-menu' ); ?>
					</label>
					<input
						data-js="advanced-sidebar-menu-pro/widget/accordion-color"
						data-target="<?php echo esc_attr( $widget->get_field_id( 'accordion-icons' ) ); ?>"
						type="text"
						name="<?php echo esc_attr( $widget->get_field_name( self::ICON_COLOR ) ); ?>"
						value="<?php echo esc_attr( $instance[ self::ICON_COLOR ] ); ?>"
						id="<?php echo esc_attr( $widget->get_field_id( self::ICON_COLOR ) ); ?>"
						class="asm-color-select">
					<br />
				</p>

				<p>
					<?php $widget->checkbox( self::KEEP_CLOSED ); ?>

					<label>
						<?php esc_html_e( 'Close all accordion sections until clicked', 'advanced-sidebar-menu' ); ?>
					</label>
					<br />
					<span class="description">
						<em><?php esc_html_e( 'If unchecked, the menu will open to the current section automatically.', 'advanced-sidebar-menu' ); ?></em>
					</span>
				</p>
				<p>
					<?php $widget->checkbox( self::INCLUDE_PARENT ); ?>
					<label>
						<?php esc_html_e( 'Include highest level parent in accordion', 'advanced-sidebar-menu' ); ?>
					</label>
					<br />
					<span class="description">
						<em><?php esc_html_e( 'If checked, the highest level parent will become part of the accordion open/close ability.', 'advanced-sidebar-menu' ); ?></em>
					</span>
				</p>
			</div>
		</div>
		<?php
	}

}
